@@ -14,7 +14,10 @@ with(sleep, compare.2.vectors(extra[group == 1], extra[group == 2]))
 ## 3  coin::Wilcoxon              Z     -1.854      NA 0.06487
 ## 4          median              Z      1.744      NA 0.17867
 
+# compare with:
+with(sleep, compare.2.vectors(extra[group == 1], extra[group == 2], alternative = "less"))
 
+with(sleep, compare.2.vectors(extra[group == 1], extra[group == 2], alternative = "greater"))
 
 # doesn't make much sense as the data is not paired, but whatever:
 with(sleep, compare.2.vectors(extra[group == 1], extra[group == 2], paired = TRUE))
